{{ DEBUG wrapper }}
{{ Can be replaced with DEBUG-NOT to prevent any DEBUGGING and remove some memory usage }}
CON

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR ' PST formmatting control
  #14, GOTOX, GOTOY, CLS

  MAX_VALX_LEN = 64                                     ' 63 chars + zero terminator

OBJ

  DEBUG  : "FullDuplexSerial"

VAR

  BYTE nstr[MAX_VALX_LEN]                               ' string for numeric data

PUB start
                          
  DEBUG.start(31, 30, 0, 57600)
  waitcnt(clkfreq >> 1 + cnt)                           ' wait for FullDuplexSerial to fully initialize

PUB start_baud (baud)
                          
  DEBUG.start(31, 30, 0, baud)
  waitcnt(clkfreq >> 1 + cnt)                           ' wait for FullDuplexSerial to fully initialize

PUB stop

  pause
  DEBUG.stop

PUB str (value)

  DEBUG.str(value)

PUB hex (value, len)

  DEBUG.hex(value, len)

PUB dec (value)

  DEBUG.dec(value)

PUB tx (value)

  DEBUG.tx(value)

PUB bin (value, len)

  DEBUG.bin(value, len)

PUB clr
'' Clear PST (CLS / $10 / 16)

  DEBUG.tx(CLS)

PUB nl
'' Insert new line (CR / $D / 13)

  DEBUG.tx(CR)

PUB mnl (amount)
'' Insert multiple new lines (CR / $D / 13)

  REPEAT amount
    DEBUG.tx(CR)

PUB pause
'' pause for long enough for cog to empty its buffer
                                                
  waitcnt(clkfreq / 500 + cnt)

PUB decx (value, len)
'' DEBUG zero padded integer

  DEBUG.str(dcx(value, len, "0", false))

PUB decpad (value, len, char, beforeNeg)
'' DEBUG padded integer
'' Use to pad a decimal value with a specific character. Great for formatted text areas. Forces output to a specific size.
'' char is the value to pad with (recomendations: "0" or " ")
'' beforeNeg determines where the padding is inserted( true for before the negation symbol, false for after)
''   example: .decpad(156, 7, " ", true)   results in  "    156"
''   example: .decpad(-156, 7, " ", true)  results in  "   -156"
''   example: .decpad(-156, 8, "0", false) results in "-0000156"
''   example: .decpad(-156, 2, " ", true)  results in       "-6"

  DEBUG.str(dcx(value, len, char, beforeNeg))

{PUB decpad2 (value, len, char, beforeNeg) | tstr[MAX_VALX_LEN / 4]

  bytemove(@tstr, decstr(value), MAX_VALX_LEN)
  'str(@tstr)
  strx(@tstr, len, char) }

PUB decstr(value) | idx, v
'' return a string representation of a number

  v := value                                            ' temperalily store value
  idx := 1
  REPEAT WHILE (v /= 10)                                ' determine number of digits
    idx++                                               ' store into idx

  IF (v := (value < 0))                                 ' negative value?  
    -value                                              ' make positive
    nstr[0] := "-"                                      ' move index one position
    idx++

  nstr[idx--] := 0                                      ' terminate string

  REPEAT idx                                            ' cycle through all digits
    nstr[idx--] := ((value // 10) + "0")                ' convert digit to ASCII
    value /= 10
  
  RETURN @nstr 

PUB decbase32 (value) | idx, v
'' return a base32 string representation of a number

  v := value                                            ' temperalily store value
  idx := 1
  REPEAT WHILE (v >>= 5)                                ' determine number of digits
    idx++                                               ' store into idx

  nstr[idx] := 0                                        ' terminate string

  value <-= 5                                           ' prep value
  REPEAT idx--
    nstr[idx--] := lookupz((value ->= 5) & $1F : "0".."9", "A".."V")

  RETURN @nstr 

PUB decbase64 (value) | idx, v
'' return a base64 string representation of a number
'' a proprietary base 64 numbering system

  v := value                                            ' temperalily store value
  idx := 1
  REPEAT WHILE (v >>= 6)                                ' determine number of digits
    idx++                                               ' store into idx

  nstr[idx] := 0                                        ' terminate string

  value <-= 6                                           ' prep value
  REPEAT idx--
    nstr[idx--] := lookupz((value ->= 6) & $3F : "0".."9", "A".."Z", "a".."z","+","/")
         
  RETURN @nstr 

{PUB decstr2(value) | idx, div, z_pad   

' Converts value to signed-decimal string equivalent
' -- characters written to current position of idx
' -- returns pointer to nstr

  bytefill(@nstr, 0, 12)                            ' clear string to zeros
  idx := 0                                                  ' reset index

  if (value < 0)                                        ' negative value? 
    -value                                              '   yes, make positive
    nstr[idx++] := "-"                                  '   and print sign indicator

  div := 1_000_000_000                                  ' initialize divisor
  z_pad~                                                ' clear zero-pad flag

  repeat 10
    if (value => div)                                   ' printable character?
      nstr[idx++] := (value / div + "0")                '   yes, print ASCII digit
      value //= div                                     '   update value
      z_pad~~                                           '   set zflag
    elseif z_pad or (div == 1)                          ' printing or last column?
      nstr[idx++] := "0"
    div /= 10 

  return @nstr
       }

PUB dcx (value, len, char, beforeNeg) | v, idx, digs
'' Use to pad a decimal value with a specific character. Great for formatted text areas
'' char is the value to pad with (recomendations: "0" or " ")
'' beforeNeg determines where the padding is inserted( true for before the negation symbol, false for after)
''   example: .dcx(-156, 7, " ", true)  results in  "   -156"
''   example: .dcx(-156, 8, "0", false) results in "-0000156"

  idx := (len := 1 #> len <# constant(MAX_VALX_LEN - 1)) - 1

  v := value
  digs := 1
  REPEAT WHILE (v /= 10)
    digs++

  nstr[len] := 0

  IF (v := (value < 0))                                 ' negative value?
    len--   
    -value

  IF (NOT(v) OR beforeNeg)
    bytefill(@nstr, char, (len - digs) #> 0)
  ELSE
    bytefill(@nstr + 1, char, (len - digs) #> 0)  

  REPEAT digs <# len
    nstr[idx--] := ((value // 10) + "0")                ' convert digit to ASCII
    value /= 10

  IF (v)                                                ' negative value? 
    IF (beforeNeg)                                                  
      nstr[idx] := "-"                              
    ELSE
      nstr[0] := "-"
  
  RETURN @nstr 

PUB dcx2 (value, len) | div, idx
  bytefill(@nstr, 0, MAX_VALX_LEN)
  idx := 0  
  len := 1 #> len <# 10

  IF (value < 0)                                        ' negative value?   
    -value                                              '   yes, make positive
    nstr[idx++] := "-"                                  '   and print sign indicator

  div := 1_000_000_000                                  ' initialize divisor
  IF (len < 10)                                         ' less than 10 digits?
    REPEAT (10 - len)                                   '   yes, adjust divisor
      div /= 10
  
  value //= (div * 10)                                  ' truncate unused digits
  
  REPEAT len
    nstr[idx++] := (value / div + "0")                  ' convert digit to ASCII
    value //= div                                       ' update value
    div /= 10                                           ' update divisor

  RETURN @nstr 

PUB strx (addr, len, char) | sl, idx
'' Pad string to a specific length

  bytefill(@nstr, 0, MAX_VALX_LEN)
  len := 1 #> len <# constant(MAX_VALX_LEN - 1)
  sl := 1 #> strsize(addr) <# constant(MAX_VALX_LEN - 1)

  bytefill(@nstr, char, idx := (len - sl) #> 0)
  bytemove(@nstr[idx], addr, sl <# len)

  DEBUG.str(@nstr)

PUB bool (value)
'' Display true or falso value

  IF (value)
    DEBUG.str(string("True"))
  ELSE
    DEBUG.str(string("False"))

PUB arrDec (addr, length, size, withIdx) | i, idxDigs
'' Cycle through an array and display each value
'' addr is the address of the array
'' length is the length of the array
'' size is the type of array (1 == BYTE, 2 == WORD, all else == LONG)
'' withIdx (true/false) will display the index in front of the value

  IF (withIdx)
    idxDigs := get_digs(--length)        

  REPEAT i FROM 0 TO length
    IF (withIdx)
      decx(i, idxDigs)
      DEBUG.str(string(": "))

    CASE size
      1: DEBUG.dec(byte[addr][i])
      2: DEBUG.dec(word[addr][i])
      OTHER: DEBUG.dec(long[addr][i])
    DEBUG.tx(CR)

PUB arrDecx (addr, vlength, arrlength, size, withIdx) | i, idxDigs
'' Cycle through an array and display each value
'' addr is the address of the array
'' vlength is the forced size of each value (for decx)
'' arrlength is the length of the array
'' size is the type of array (1 == BYTE, 2 == WORD, all else == LONG)
'' withIdx (true/false) will display the index in front of the value

  IF (withIdx)
    idxDigs := get_digs(--arrlength)        

  REPEAT i FROM 0 TO arrlength
    IF (withIdx)
      decx(i, idxDigs)
      DEBUG.str(string(": "))

    CASE size
      1: decx(byte[addr][i], vlength)
      2: decx(word[addr][i], vlength)
      OTHER: decx(long[addr][i], vlength)
    DEBUG.tx(CR)

PUB arrBin (addr, vlength, arrlength, size, withIdx) | i, idxDigs
'' Cycle through an array and display each value
'' addr is the address of the array
'' vlength is the forced size of each value (for bin)
'' arrlength is the length of the array
'' size is the type of array (1 == BYTE, 2 == WORD, all else == LONG)
'' withIdx (true/false) will display the index in front of the value

  IF (withIdx)
    idxDigs := get_digs(--arrlength)        

  REPEAT i FROM 0 TO arrlength
    IF (withIdx)
      decx(i, idxDigs)
      DEBUG.str(string(": "))

    CASE size
      1: bin(byte[addr][i], vlength)
      2: bin(word[addr][i], vlength)
      OTHER: bin(long[addr][i], vlength)
    DEBUG.tx(CR)

PUB arrHex (addr, vlength, arrlength, size, withIdx) | i, idxDigs
'' Cycle through an array and display each value
'' addr is the address of the array
'' vlength is the forced size of each value (for hex)
'' arrlength is the length of the array
'' size is the type of array (1 == BYTE, 2 == WORD, all else == LONG)
'' withIdx (true/false) will display the index in front of the value

  IF (withIdx)
    idxDigs := get_digs(--arrlength)        

  REPEAT i FROM 0 TO arrlength
    IF (withIdx)
      decx(i, idxDigs)
      DEBUG.str(string(": "))

    CASE size
      1: hex(byte[addr][i], vlength)
      2: hex(word[addr][i], vlength)
      OTHER: hex(long[addr][i], vlength)
    DEBUG.tx(CR)

PRI get_digs (len) : idxDigs

  idxDigs := 1
  REPEAT WHILE (len /= 10)
    idxDigs++

PUB volt (v, strAddr)
'' display voltage, supply voltage as V * 100 (i.e. 12.03V should be supplied as 1203)
                 
  DEBUG.str(strAddr)
  DEBUG.tx(":")  
  decpad(v / 100, 2, " ", true)
  DEBUG.tx(".")
  decx(v // 100, 2)
  DEBUG.str(string("V "))

PUB temp (v)
'' display temperature, supply temperature as T * 10 (i.e. 72.3F should be supplied as 723)
                           
  decpad(v / 10, 3, " ", true)
  DEBUG.tx(".")
  DEBUG.dec(||v // 10)
  DEBUG.str(string("F "))