CON

  {{== EEPROM Constants ==}}
  BootPin     = 28              ' I2C Boot EEPROM SCL Pin
  EEPROM      = $A0             ' I2C EEPROM Device Address

  page_size   = 64

OBJ

  i2c   : "BASIC_I2C_Driver"    ' for read/write operations to EEPROM
  'i2c   : "PASM_I2C_Driver"     ' for read/write operations to EEPROM

VAR

  BYTE val[page_size]           ' store page-size read values

PUB init
'' initiate i2c object and setup pointers for this object
'' returns true if EEPROM has already been initialized, false if it was not
  
  i2c.Initialize(BootPin)                                                       ' start i2c object                                                            
      
PUB write_multipage (eepromAddr, dataAddr, size) | i
'' write an array (that spans more than one page) to EEPROM over multiple pages
'' size is the number of bytes to be stored 

  i := page_size - eepromAddr // page_size
  IF (size =< i)                                                                ' if amount of info is less than what's left in this page
    write(eepromAddr, dataAddr, size)
    RETURN true
  
  write(eepromAddr, dataAddr, i) 
  size -= i                                                                     ' remaining data
  REPEAT
    IF (size =< page_size)
      write(eepromAddr + i, dataAddr + i, size)
      QUIT
    ELSE
      IF (write(eepromAddr + i, dataAddr + i, page_size))
        i += page_size
        size -= page_size
      ELSE
        RETURN false

  RETURN true

PUB read_multipage (eepromAddr, size) : mem_addr | i
'' read an array (that spans more than one page) from EEPROM, store it in highest location possible in RAM
'' size is the number of bytes to be read

  i := page_size - eepromAddr // page_size                                           ' determine bytes to end of page
  mem_addr := $7FFF - size                                                    ' start address
  IF (size =< i)
    bytemove(mem_addr, read(eepromAddr, size), size)
    RETURN  
   
  bytemove(mem_addr, read(eepromAddr, i), i)                                         ' move to lowest portion of the highest location in RAM
  size -= i
  eepromAddr += i                                                                    ' remaining string size
  REPEAT
    IF (size =< page_size)
      bytemove($7FFF - size, read(eepromAddr, size), size)                           ' read the last of the info
      QUIT
    ELSE
      bytemove($7FFF - size, read(eepromAddr, page_size), page_size)                 ' read a page of the info
      eepromAddr += page_size
      size -= page_size

PUB write (eepromAddr, valueAddr, size) | time
'' write page to EEPROM with watchdog
'' size is the number of bytes to be stored  

  IF (i2c.WritePage(BootPin, EEPROM, eepromAddr, valueAddr, size))
    RETURN false
  time := cnt
  REPEAT WHILE (i2c.WriteWait(BootPin, EEPROM, eepromAddr))                           ' wait for watchdog
    IF (cnt - time > clkfreq / 10)
      RETURN false

  RETURN true 
 
PUB read (eepromAddr, size)
'' read page from EEPROM, return address of value
'' size is the number of bytes to be read
                          
  IF i2c.ReadPage(BootPin, EEPROM, eepromAddr, @val, size)
    RETURN false                     

  RETURN @val
